;; jbank@alum.mit.edu
;; 10/2000
;;
;; Modification to shell mode to automatically cache passwords.
;; This has the effect similar to sudo, with password caching
;; on, emacs keeps track of the last password you typed and 
;; responds with that password automatically till it times out.
;;
;; Exports: 
;;  Variables: password-cache-timeout

(autoload 'comint-check-proc "comint")
(autoload 'shell-mode "shell")

(require 'shell)
(require 'timer)

(setq comint-output-filter-functions
      (append comint-output-filter-functions
              (list 'comint-watch-for-cached-password)))

(defcustom password-cache-timeout 300 "Time till shell password cache expires")
(defcustom cached-password nil "Controls whether passwords are cached or not")

(defvar last-timer nil)
(defun password-cache-off ()
  (interactive)
  (setq password-cache-timeout 0))

(defun password-cache-on ()
  (interactive)
  (setq password-cache-timeout 300))

(defun uncache-password ()
  (interactive)
  (setq cached-password nil))

(defun update-password-cache (str) 
  (if (not (null last-timer)) (cancel-timer last-timer))
  (setq last-timer (run-with-timer password-cache-timeout nil 'uncache-password))
  (setq cached-password str))

(defun send-invisible-cached (str)
  "Read a string without echoing.
Then send it to the process running in the current buffer.
The string is sent using `comint-input-sender'.
Security bug: your string can still be temporarily recovered with
\\[view-lossage]."
  (interactive "P")			; Defeat snooping via C-x ESC ESC
  (update-password-cache str)
  (let ((proc (get-buffer-process (current-buffer))))
    (cond ((not proc)
	   (error "Current buffer has no process"))
	  ((stringp str)
	   (funcall comint-input-sender proc str))
	  (t
	   (let ((str (comint-read-noecho "Non-echoed text: " t)))
	     (if (stringp str)
		 (send-invisible-cached str)
	       (message "Warning: text will be echoed")))))))

(defun comint-watch-for-cached-password (string) 
  "Prompt in the minibuffer for password and send without echoing.
This function uses `send-invisible' to read and send a password to the buffer's
process if STRING contains a password prompt defined by 
`comint-password-prompt-regexp'.

This function could be in the list `comint-output-filter-functions'."
  (if (string-match comint-password-prompt-regexp string)
      (send-invisible-cached cached-password))
  ;;we also watch for denials, may want to improve this regexp.
  (if (string-match "Permission denied." string) 
      (uncache-password))
)
